@@ -9,10 +9,11 @@ use mro 'c3';
 
 use DBIx::Class::Carp;
 use Scalar::Util qw/refaddr weaken reftype blessed/;
+use List::Util qw/first/;
 use Context::Preserve 'preserve_context';
 use Try::Tiny;
-use SQL::Abstract::Util qw(is_plain_value is_literal_value);
-use DBIx::Class::_Util qw(quote_sub perlstring serialize detected_reinvoked_destructor);
+use SQL::Abstract qw(is_plain_value is_literal_value);
+use DBIx::Class::_Util qw(quote_sub perlstring serialize);
 use namespace::clean;
 
 # default cursor class, overridable in connect_info attributes
@@ -252,8 +253,6 @@ sub new {
 }
 
 sub DESTROY {
-  return if &detected_reinvoked_destructor;
-
   $_[0]->_verify_pid unless DBIx::Class::_ENV_::BROKEN_FORK;
   # some databases spew warnings on implicit disconnect
   local $SIG{__WARN__} = sub {};
@@ -1060,45 +1059,11 @@ sub _populate_dbh {
 }
 
 sub _run_connection_actions {
-  # there are pathological cases in the CI where this can loop
-  # did not investigae in depth, but in either case this makes
-  # sense to guard like this
-  return if $_[0]->{_running_connections_actions};
-
-  local $_[0]->{_running_connections_actions} = 1;
 
   $_[0]->_do_connection_actions(connect_call_ => $_) for (
     ( $_[0]->on_connect_call || () ),
     $_[0]->_parse_connect_do ('on_connect_do'),
   );
-
-  my $sqlac_like;
-  if(
-    DBIx::Class::_ENV_::DEVREL
-      and
-    $ENV{DBICDEVREL_SWAPOUT_SQLAC_WITH}
-      and
-    ( $sqlac_like ) = $ENV{DBICDEVREL_SWAPOUT_SQLAC_WITH} =~ /(.+)/
-      and
-    # delay calling ->sql_maker as long as we can
-    # ensure_class_loaded returns undef or throws
-    ( Class::C3::Componentised->ensure_class_loaded( $sqlac_like ), 1 )
-      and
-    ( ref $_[0]->sql_maker ) !~ /__REBASED__/
-  ) {
-
-    require DBIx::Class::SQLMaker::ClassicExtensions;
-    require SQL::Abstract::Classic;
-
-    Class::C3::Componentised->inject_base(
-      'DBICDevRel::SQLAC::SwapOut',
-      'DBIx::Class::SQLMaker::ClassicExtensions',
-      $sqlac_like,
-      'SQL::Abstract::Classic',
-    );
-
-    $_[0]->_do_connection_actions(connect_call_ => [[ rebase_sqlmaker => 'DBICDevRel::SQLAC::SwapOut' ]]);
-  }
 }
 
 
@@ -1147,9 +1112,6 @@ sub _server_info {
   unless ($info = $self->_dbh_details->{info}) {
 
     $info = {};
-    # this guarantees that problematic conninfo won't be hidden
-    # by the try{} below
-    $self->ensure_connected;
 
     my $server_version = try {
       $self->_get_server_version
@@ -1357,16 +1319,7 @@ sub _extract_driver_from_connect_info {
     # try to use dsn to not require being connected, the driver may still
     # force a connection later in _rebless to determine version
     # (dsn may not be supplied at all if all we do is make a mock-schema)
-    #
-    # Use the same regex as the one used by DBI itself (even if the use of
-    # \w is odd given unicode):
-    # https://metacpan.org/source/TIMB/DBI-1.634/DBI.pm#L621
-    #
-    # DO NOT use https://metacpan.org/source/TIMB/DBI-1.634/DBI.pm#L559-566
-    # as there is a long-standing precedent of not loading DBI.pm until the
-    # very moment we are actually connecting
-    #
-    ($drv) = ($self->_dbi_connect_info->[0] || '') =~ /^dbi:(\w*)/i;
+    ($drv) = ($self->_dbi_connect_info->[0] || '') =~ /^dbi:([^:]+):/i;
     $drv ||= $ENV{DBI_DRIVER};
   }
 
@@ -1416,40 +1369,24 @@ sub _warn_undetermined_driver {
 }
 
 sub _do_connection_actions {
-  my ($self, $method_prefix, $call, @args) = @_;
+  my $self          = shift;
+  my $method_prefix = shift;
+  my $call          = shift;
 
-  try {
-    if (not ref($call)) {
-      my $method = $method_prefix . $call;
-      $self->$method(@args);
-    }
-    elsif (ref($call) eq 'CODE') {
-      $self->$call(@args);
-    }
-    elsif (ref($call) eq 'ARRAY') {
-      if (ref($call->[0]) ne 'ARRAY') {
-        $self->_do_connection_actions($method_prefix, $_) for @$call;
-      }
-      else {
-        $self->_do_connection_actions($method_prefix, @$_) for @$call;
-      }
-    }
-    else {
-      $self->throw_exception (sprintf ("Don't know how to process conection actions of type '%s'", ref($call)) );
+  if (not ref($call)) {
+    my $method = $method_prefix . $call;
+    $self->$method(@_);
+  } elsif (ref($call) eq 'CODE') {
+    $self->$call(@_);
+  } elsif (ref($call) eq 'ARRAY') {
+    if (ref($call->[0]) ne 'ARRAY') {
+      $self->_do_connection_actions($method_prefix, $_) for @$call;
+    } else {
+      $self->_do_connection_actions($method_prefix, @$_) for @$call;
     }
+  } else {
+    $self->throw_exception (sprintf ("Don't know how to process conection actions of type '%s'", ref($call)) );
   }
-  catch {
-    if ( $method_prefix =~ /^connect/ ) {
-      # this is an on_connect cycle - we can't just throw while leaving
-      # a handle in an undefined state in our storage object
-      # kill it with fire and rethrow
-      $self->_dbh(undef);
-      $self->throw_exception( $_[0] );
-    }
-    else {
-      carp "Disconnect action failed: $_[0]";
-    }
-  };
 
   return $self;
 }
@@ -1506,92 +1443,6 @@ sub _do_query {
   return $self;
 }
 
-=head2 connect_call_rebase_sqlmaker
-
-This on-connect call takes as a single argument the name of a class to "rebase"
-the SQLMaker inheritance hierarchy upon. For this to work properly the target
-class B<MUST> inherit from L<DBIx::Class::SQLMaker::ClassicExtensions> and
-L<SQL::Abstract::Classic> as shown below.
-
-This infrastructure is provided to aid recent activity around experimental new
-aproaches to SQL generation within DBIx::Class. You can (and are encouraged to)
-mix and match old and new within the same codebase as follows:
-
-  package DBIx::Class::Awesomer::SQLMaker;
-  # you MUST inherit in this order to get the composition right
-  # you are free to override-without-next::method any part you need
-  use base qw(
-    DBIx::Class::SQLMaker::ClassicExtensions
-    << OPTIONAL::AWESOME::Class::Implementing::ExtraRainbowSauce >>
-    SQL::Abstract::Classic
-  );
-  << your new code goes here >>
-
-
-  ... and then ...
-
-
-  my $experimental_schema = $original_schema->connect(
-    sub {
-      $original_schema->storage->dbh
-    },
-    {
-      # the nested arrayref is important, as per
-      # https://metacpan.org/pod/DBIx::Class::Storage::DBI#on_connect_call
-      on_connect_call => [ [ rebase_sqlmaker => 'DBIx::Class::Awesomer::SQLMaker' ] ],
-    },
-  );
-
-=cut
-
-sub connect_call_rebase_sqlmaker {
-  my( $self, $requested_base_class ) = @_;
-
-  $self->throw_exception(
-    "The on_connect callee 'rebase_sqlmaker' expects a single plain string argument: the name of the target base class"
-  ) if (
-    @_ != 2
-      or
-    ! length( $requested_base_class )
-  );
-
-  my $old_class = ref( $self->sql_maker );
-
-  # nothing to do!
-  return if $old_class->isa( $requested_base_class );
-
-  my $synthetic_class = "${old_class}__REBASED_ON__${requested_base_class}";
-
-  {
-    no strict 'refs';
-
-    # skip if we already made that class
-    unless( @{"${synthetic_class}::ISA"} ) {
-
-      $self->ensure_class_loaded( $requested_base_class );
-
-      for my $base (qw(
-        DBIx::Class::SQLMaker::ClassicExtensions
-        SQL::Abstract::Classic
-      )) {
-
-        $self->throw_exception(
-          "The 'rebase_sqlmaker' target class '$requested_base_class' is not inheriting from '$base', this can not work"
-        ) unless $requested_base_class->isa( $base );
-      }
-
-      $self->inject_base( $synthetic_class, $old_class, $requested_base_class );
-
-      Class::C3->reinitialize
-        if DBIx::Class::_ENV_::OLD_MRO;
-    }
-  }
-
-  # force re-build on next access for this particular $storage instance
-  $self->sql_maker_class( $synthetic_class );
-  $self->_sql_maker( undef );
-}
-
 sub _connect {
   my $self = shift;
 
@@ -1806,7 +1657,7 @@ sub _gen_sql_bind {
       and
     $op eq 'select'
       and
-    grep {
+    first {
       length ref $_->[1]
         and
       blessed($_->[1])
@@ -1816,8 +1667,8 @@ sub _gen_sql_bind {
   ) {
     carp_unique 'DateTime objects passed to search() are not supported '
       . 'properly (InflateColumn::DateTime formats and settings are not '
-      . 'respected.) See ".. format a DateTime object for searching?" in '
-      . 'DBIx::Class::Manual::FAQ. To disable this warning for good '
+      . 'respected.) See "Formatting DateTime objects in queries" in '
+      . 'DBIx::Class::Manual::Cookbook. To disable this warning for good '
       . 'set $ENV{DBIC_DT_SEARCH_OK} to true'
   }
 
@@ -1847,6 +1698,7 @@ sub _resolve_bindattrs {
   };
 
   return [ map {
+    my $resolved =
       ( ref $_ ne 'ARRAY' or @$_ != 2 ) ? [ {}, $_ ]
     : ( ! defined $_->[0] )             ? [ {}, $_->[1] ]
     : (ref $_->[0] eq 'HASH')           ? [(
@@ -1863,6 +1715,31 @@ sub _resolve_bindattrs {
     :                                     [ $resolve_bindinfo->(
                                               { dbic_colname => $_->[0] }
                                             ), $_->[1] ]
+    ;
+
+    if (
+      ! exists $resolved->[0]{dbd_attrs}
+        and
+      ! $resolved->[0]{sqlt_datatype}
+        and
+      length ref $resolved->[1]
+        and
+      ! is_plain_value $resolved->[1]
+    ) {
+      require Data::Dumper;
+      local $Data::Dumper::Maxdepth = 1;
+      local $Data::Dumper::Terse = 1;
+      local $Data::Dumper::Useqq = 1;
+      local $Data::Dumper::Indent = 0;
+      local $Data::Dumper::Pad = ' ';
+      $self->throw_exception(
+        'You must supply a datatype/bindtype (see DBIx::Class::ResultSet/DBIC BIND VALUES) '
+      . 'for non-scalar value '. Data::Dumper::Dumper ($resolved->[1])
+      );
+    }
+
+    $resolved;
+
   } @$bind ];
 }
 
@@ -1896,31 +1773,28 @@ sub _query_end {
 }
 
 sub _dbi_attrs_for_bind {
-  my ($self, $ident, $bind) = @_;
+  #my ($self, $ident, $bind) = @_;
 
-  my @attrs;
+  return [ map {
 
-  for (map { $_->[0] } @$bind) {
-    push @attrs, do {
-      if (exists $_->{dbd_attrs}) {
-        $_->{dbd_attrs}
-      }
-      elsif($_->{sqlt_datatype}) {
-        # cache the result in the dbh_details hash, as it can not change unless
-        # we connect to something else
-        my $cache = $self->_dbh_details->{_datatype_map_cache} ||= {};
-        if (not exists $cache->{$_->{sqlt_datatype}}) {
-          $cache->{$_->{sqlt_datatype}} = $self->bind_attribute_by_data_type($_->{sqlt_datatype}) || undef;
-        }
-        $cache->{$_->{sqlt_datatype}};
-      }
-      else {
-        undef;  # always push something at this position
-      }
-    }
-  }
+    exists $_->{dbd_attrs}  ?  $_->{dbd_attrs}
 
-  return \@attrs;
+  : ! $_->{sqlt_datatype}   ? undef
+
+  :                           do {
+
+    # cache the result in the dbh_details hash, as it (usually) can not change
+    # unless we connect to something else
+    # FIXME: for the time being Oracle is an exception, pending a rewrite of
+    # the LOB storage
+    my $cache = $_[0]->_dbh_details->{_datatype_map_cache} ||= {};
+
+    $cache->{$_->{sqlt_datatype}} = $_[0]->bind_attribute_by_data_type($_->{sqlt_datatype})
+      if ! exists $cache->{$_->{sqlt_datatype}};
+
+    $cache->{$_->{sqlt_datatype}};
+
+  } } map { $_->[0] } @{$_[2]} ];
 }
 
 sub _execute {
@@ -2096,30 +1970,12 @@ sub insert {
 
   my %returned_cols = %$to_insert;
   if (my $retlist = $sqla_opts->{returning}) {  # if IR is supported - we will get everything in one set
-
-    unless( @ir_container ) {
-      try {
-
-        # FIXME - need to investigate why Caelum silenced this in 4d4dc518
-        local $SIG{__WARN__} = sub {};
-
-        @ir_container = $sth->fetchrow_array;
-        $sth->finish;
-
-      } catch {
-        # Evict the $sth from the cache in case we got here, since the finish()
-        # is crucial, at least on older Firebirds, possibly on other engines too
-        #
-        # It would be too complex to make this a proper subclass override,
-        # and besides we already take the try{} penalty, adding a catch that
-        # triggers infrequently is a no-brainer
-        #
-        if( my $kids = $self->_dbh->{CachedKids} ) {
-          $kids->{$_} == $sth and delete $kids->{$_}
-            for keys %$kids
-        }
-      };
-    }
+    @ir_container = try {
+      local $SIG{__WARN__} = sub {};
+      my @r = $sth->fetchrow_array;
+      $sth->finish;
+      @r;
+    } unless @ir_container;
 
     @returned_cols{@$retlist} = @ir_container if @ir_container;
   }
@@ -2207,7 +2063,7 @@ sub _insert_bulk {
   # because a user-supplied literal/bind (or something else specific to a
   # resultsource and/or storage driver) can inject extra binds along the
   # way, so one can't rely on "shift positions" ordering at all. Also we
-  # can't just hand SQLMaker a set of some known "values" (e.g. hashrefs that
+  # can't just hand SQLA a set of some known "values" (e.g. hashrefs that
   # can be later matched up by address), because we want to supply a real
   # value on which perhaps e.g. datatype checks will be performed
   my ($proto_data, $serialized_bind_type_by_col_idx);
@@ -2555,7 +2411,7 @@ sub _select_args {
   # however currently we *may* pass the same $orig_attrs
   # with different ident/select/where
   # the whole interface needs to be rethought, since it
-  # was centered around the flawed SQLMaker API. We can do
+  # was centered around the flawed SQLA API. We can do
   # soooooo much better now. But that is also another
   # battle...
   #return (
@@ -2725,9 +2581,9 @@ see L<DBIx::Class::SQLMaker::LimitDialects>.
 sub _dbh_columns_info_for {
   my ($self, $dbh, $table) = @_;
 
-  if ($dbh->can('column_info')) {
-    my %result;
-    my $caught;
+  my %result;
+
+  if (! DBIx::Class::_ENV_::STRESSTEST_COLUMN_INFO_UNAWARE_STORAGE and $dbh->can('column_info')) {
     try {
       my ($schema,$tab) = $table =~ /^(.+?)\.(.+)$/ ? ($1,$2) : (undef,$table);
       my $sth = $dbh->column_info( undef,$schema, $tab, '%' );
@@ -2744,39 +2600,75 @@ sub _dbh_columns_info_for {
         $result{$col_name} = \%column_info;
       }
     } catch {
-      $caught = 1;
+      %result = ();
     };
-    return \%result if !$caught && scalar keys %result;
+
+    return \%result if keys %result;
   }
 
-  my %result;
   my $sth = $dbh->prepare($self->sql_maker->select($table, undef, \'1 = 0'));
   $sth->execute;
-  my @columns = @{$sth->{NAME_lc}};
-  for my $i ( 0 .. $#columns ){
-    my %column_info;
-    $column_info{data_type} = $sth->{TYPE}->[$i];
-    $column_info{size} = $sth->{PRECISION}->[$i];
-    $column_info{is_nullable} = $sth->{NULLABLE}->[$i] ? 1 : 0;
 
-    if ($column_info{data_type} =~ m/^(.*?)\((.*?)\)$/) {
-      $column_info{data_type} = $1;
-      $column_info{size}    = $2;
+### The acrobatics with lc names is necessary to support both the legacy
+### API that used NAME_lc exclusively, *AND* at the same time work properly
+### with column names differing in cas eonly (thanks pg!)
+
+  my ($columns, $seen_lcs);
+
+  ++$seen_lcs->{lc($_)} and $columns->{$_} = {
+    idx => scalar keys %$columns,
+    name => $_,
+    lc_name => lc($_),
+  } for @{$sth->{NAME}};
+
+  $seen_lcs->{$_->{lc_name}} == 1
+    and
+  $_->{name} = $_->{lc_name}
+    for values %$columns;
+
+  for ( values %$columns ) {
+    my $inf = {
+      data_type => $sth->{TYPE}->[$_->{idx}],
+      size => $sth->{PRECISION}->[$_->{idx}],
+      is_nullable => $sth->{NULLABLE}->[$_->{idx}] ? 1 : 0,
+    };
+
+    if ($inf->{data_type} =~ m/^(.*?)\((.*?)\)$/) {
+      @{$inf}{qw( data_type  size)} = ($1, $2);
     }
 
-    $result{$columns[$i]} = \%column_info;
+    $result{$_->{name}} = $inf;
   }
+
   $sth->finish;
 
-  foreach my $col (keys %result) {
-    my $colinfo = $result{$col};
-    my $type_num = $colinfo->{data_type};
-    my $type_name;
-    if(defined $type_num && $dbh->can('type_info')) {
-      my $type_info = $dbh->type_info($type_num);
-      $type_name = $type_info->{TYPE_NAME} if $type_info;
-      $colinfo->{data_type} = $type_name if $type_name;
+  if ($dbh->can('type_info')) {
+    for my $inf (values %result) {
+      next if ! defined $inf->{data_type};
+
+      $inf->{data_type} = (
+        (
+          (
+            $dbh->type_info( $inf->{data_type} )
+              ||
+            next
+          )
+            ||
+          next
+        )->{TYPE_NAME}
+          ||
+        next
+      );
+
+      # FIXME - this may be an artifact of the DBD::Pg implmentation alone
+      # needs more testing in the future...
+      $inf->{size} -= 4 if (
+        ( $inf->{size}||0 > 4 )
+          and
+        $inf->{data_type} =~ qr/^text$/i
+      );
     }
+
   }
 
   return \%result;
@@ -3012,8 +2904,8 @@ sub create_ddl_dir {
     %{$sqltargs || {}}
   };
 
-  unless (DBIx::Class::Optional::Dependencies->req_ok_for ('deploy')) {
-    $self->throw_exception("Can't create a ddl file without " . DBIx::Class::Optional::Dependencies->req_missing_for ('deploy') );
+  if (my $missing = DBIx::Class::Optional::Dependencies->req_missing_for ('deploy')) {
+    $self->throw_exception("Can't create a ddl file without $missing");
   }
 
   my $sqlt = SQL::Translator->new( $sqltargs );
@@ -3169,8 +3061,8 @@ sub deployment_statements {
       return join('', @rows);
   }
 
-  unless (DBIx::Class::Optional::Dependencies->req_ok_for ('deploy') ) {
-    $self->throw_exception("Can't deploy without a ddl_dir or " . DBIx::Class::Optional::Dependencies->req_missing_for ('deploy') );
+  if (my $missing = DBIx::Class::Optional::Dependencies->req_missing_for ('deploy') ) {
+    $self->throw_exception("Can't deploy without a pregenerated 'ddl_dir' directory or $missing");
   }
 
   # sources needs to be a parser arg, but for simplicity allow at top level
